<?php

namespace Drupal\openstory\Plugin\rest\resource;

use Drupal\Core\File\FileSystemInterface;
use Drupal\rest\Plugin\ResourceBase;
use Drupal\rest\ResourceResponse;
use Psr\Log\LoggerInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;

/**
 * Provides resources to work with files.
 *
 * @RestResource(
 *   id = "custom_service_file",
 *   label = @Translation("File Service"),
 *   uri_paths = {
 *     "canonical" = "/custom_file/file/{fileName}",
 *     "https://www.drupal.org/link-relations/create" = "/custom_file/file"
 *   }
 * )
 */
class FileService extends ResourceBase {

  /**
   * A current user instance.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * Constructs a Drupal\rest\Plugin\ResourceBase object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param array $serializer_formats
   *   The available serialization formats.
   * @param \Psr\Log\LoggerInterface $logger
   *   A logger instance.
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   A current user instance.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system service.
   */
  public function __construct(
      array $configuration,
      $plugin_id,
      $plugin_definition,
      array $serializer_formats,
      LoggerInterface $logger,
      AccountProxyInterface $current_user,
      FileSystemInterface $file_system) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $serializer_formats, $logger);
    $this->currentUser = $current_user;
    $this->fileSystem = $file_system;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
        $configuration,
        $plugin_id,
        $plugin_definition,
        $container->getParameter('serializer.formats'),
        $container->get('logger.factory')->get('custom_rest'),
        $container->get('current_user'),
        $container->get('file_system')
    );
  }

  /**
   * Responds to entity POST requests and saves the new entity.
   *
   * @param string $data
   *   Data.
   *
   * @return \Drupal\rest\ModifiedResourceResponse
   *   The HTTP response object.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\HttpException
   */
  public function post($data) {

    if (!empty($data['filename'][0] && !empty($data['data'][0]))) {
      $fileData = $data['data'][0]['value'];
      $fileName = $data['filename'][0]['value'];
      $this->createFileTemporary($fileName, $fileData);
      return new ResourceResponse($fileName, 200);
    }
    else {
      throw new BadRequestHttpException('Must be provided file name and data.');
    }

  }

  /**
   * Responds to entity PATCH requests.
   *
   * @param string $fileName
   *   The file name.
   * @param string $data
   *   Data.
   *
   * @return \Drupal\rest\ResourceResponse
   *   The HTTP response object.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\HttpException
   */
  public function patch($fileName, $data) {

    if (!empty($fileName) && !empty($data['data'][0])) {
      $fileData = $data['data'][0]['value'];
      $fileExists = $this->checkFileExists($fileName);
      if ($fileExists) {
        $this->createFileTemporary($fileName, $fileData);
      }
      else {
        $this->createFileTemporary($fileName, $fileData);
      }
      return new ResourceResponse($fileName, 200);
    }
    else {
      throw new BadRequestHttpException('Must be provided file name and data.');
    }

  }

  /**
   * Responds to DELETE requests.
   *
   * @param string $fileName
   *   The file name.
   *
   * @return \Drupal\rest\ResourceResponse
   *   The HTTP response object.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\HttpException
   */
  public function delete($fileName) {
    if (!empty($fileName)) {
      if ($this->checkFileExists($fileName)) {
        $dir = $this->fileSystem->realpath(file_default_scheme() . "://");
        $file = $dir . '/mediaLibrary/' . $fileName;
        file_unmanaged_delete($file);
        $response = new ResourceResponse(NULL, 204);
        $response->addCacheableDependency($fileName);
        return $response;
      }
      else {
        throw new BadRequestHttpException('The file does not exists.');
      }
    }
    else {
      throw new BadRequestHttpException('Must be provided the file name.');
    }
  }

  /**
   * Check if entity file exists.
   *
   * @param string $fileName
   *   The file name.
   *
   * @return bool
   *   Return if file exists or not.
   */
  public function checkFileExists($fileName) {
    $dir = $this->fileSystem->realpath(file_default_scheme() . "://");
    $file = $dir . '/mediaLibrary/' . $fileName;
    if (!file_exists($file)) {
      return FALSE;
    }
    return TRUE;
  }

  /**
   * Check if entity file exists.
   *
   * @param string $fileName
   *   The file name.
   * @param string $fileData
   *   Data.
   */
  public function createFileTemporary($fileName, $fileData) {
    $file_contents = base64_decode($fileData);
    $dir = "public://mediaLibrary";
    if (!is_dir($dir)) {
      mkdir($dir);
    }
    $destination = $dir . '/' . $fileName;
    file_unmanaged_save_data($file_contents, $destination, FILE_EXISTS_REPLACE);
  }

}
