<?php

namespace Drupal\openstory\Plugin\rest\resource;

use Drupal\openstory\Plugin\rest\EntityTypeResourceBase;
use Drupal\rest\ResourceResponse;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Provides a resource to get the structure of an entity.
 *
 * @RestResource(
 *   id = "entity_structure",
 *   label = @Translation("Get Entity Structure File"),
 *   uri_paths = {
 *     "canonical" =
 *   "/entity-structure/{entity_type}/{bundle}",
 *     "https://www.drupal.org/link-relations/create" =
 *   "/entity-structure/{entity_type}/{bundle}"
 *   }
 * )
 */
class GetEntityStructureFile extends EntityTypeResourceBase {

  /**
   * Responds to GET requests.
   *
   * Returns a list of all fields for specified entity.
   *
   * @param string $entity_type
   *   The entity type.
   * @param string $bundle
   *   The entity bundle.
   *
   * @return \Drupal\rest\ResourceResponse
   *   The response containing entity fields.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\HttpException
   */
  public function get($entity_type, $bundle) {
    if (!empty($entity_type) && !empty($bundle)) {
      if (!$this->entityTypeManager->hasDefinition($entity_type)) {
        throw new NotFoundHttpException($this->t('No entity type found: @type.', ['@type' => $entity_type]));
      }
      $entity_structure = \Drupal::service('entity_structure');
      // Check if user has permission to create nodes, comments, etc.
      if ($this->currentUser->hasPermission($entity_structure->getUserPermissionName($entity_type, $bundle))) {
        return new ResourceResponse($this->getBundleInfo($entity_type, $bundle));
      }
      else {
        throw new AccessDeniedHttpException('Access denied. You do not have the right permission.');
      }
    }
    else {
      throw new BadRequestHttpException('Parameters entity type and bundle are mandatory.');
    }
  }

  /**
   * Gets information about the bundle.
   *
   * @param string $entity_type
   *   The entity type id.
   * @param string $bundle
   *   The bundle name.
   *
   * @return array
   *   The bundle info.
   */
  protected function getBundleInfo($entity_type, $bundle) {
    $entity = $this->entityTypeManager->getDefinition($entity_type);
    if ($bundle_entity_type_id = $entity->getBundleEntityType()) {
      $entityBundle = $this->entityTypeManager->getStorage($bundle_entity_type_id)->load($bundle);
      if (!$entityBundle) {
        throw new NotFoundHttpException(
          $this->t(
            'No bundle "@bundle" found for entity type @type',
            ['@type' => $entity_type, '@bundle' => $bundle]
          )
        );
      }
      $bundle_info['label'] = $entityBundle->label();
      $bundle_info['fields'] = $this->getBundleFields($entity_type, $bundle);
      return $bundle_info;
    }
    else {
      $bundle_info['label'] = $entity->getLabel();
      $bundle_info['fields'] = $this->getBundleFields($entity_type, $bundle);
      return $bundle_info;
    }
  }

}
