<?php

namespace Drupal\openstory\Plugin\rest\resource;

use Drupal\Core\File\FileSystemInterface;
use Drupal\rest\Plugin\ResourceBase;
use Drupal\rest\ResourceResponse;
use Psr\Log\LoggerInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides resources to work with files.
 *
 * @RestResource(
 *   id = "get_file_details",
 *   label = @Translation("Get File Details"),
 *   uri_paths = {
 *     "canonical" = "/get_file",
 *     "https://www.drupal.org/link-relations/create" = "/get_file"
 *   }
 * )
 */
class GetFileDetailsResource extends ResourceBase {

  /**
   * A current user instance.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * Constructs a Drupal\rest\Plugin\ResourceBase object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param array $serializer_formats
   *   The available serialization formats.
   * @param \Psr\Log\LoggerInterface $logger
   *   A logger instance.
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   A current user instance.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system service.
   */
  public function __construct(
      array $configuration,
      $plugin_id,
      $plugin_definition,
      array $serializer_formats,
      LoggerInterface $logger,
      AccountProxyInterface $current_user,
      FileSystemInterface $file_system) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $serializer_formats, $logger);
    $this->currentUser = $current_user;
    $this->fileSystem = $file_system;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
        $configuration,
        $plugin_id,
        $plugin_definition,
        $container->getParameter('serializer.formats'),
        $container->get('logger.factory')->get('custom_rest'),
        $container->get('current_user'),
        $container->get('file_system')
    );
  }

  /**
   * Responds to GET requests.
   *
   * @return \Drupal\rest\ResourceResponse
   *   Returns a list of of files inside mediaLibrary folder.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\HttpException
   *   Throws exception expected.
   */
  public function get() {
    $files = [];

    $filesFolder = $this->fileSystem->realpath(file_default_scheme() . "://");
    $dir = $filesFolder . '/mediaLibrary';
    foreach (new \DirectoryIterator($dir) as $file => $fileInfo) {
      $files[$file]['filename'] = $fileInfo->getFilename();
      $files[$file]['filepath'] = $fileInfo->getPathName();
      $files[$file]['size'] = $fileInfo->getSize();
      $files[$file]['filemime'] = $fileInfo->getExtension();
      $files[$file]['created'] = $fileInfo->getCTime();
    }

    return new ResourceResponse($files, 200);

  }

}
