<?php

namespace Drupal\openstory\Plugin\rest\resource;

use Drupal\Core\Session\AccountProxyInterface;
use Drupal\rest\Plugin\ResourceBase;
use Drupal\rest\ResourceResponse;
use Drupal\search_api\Entity\Index;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Search word in the site.
 *
 * @RestResource(
 *   id = "search_resource",
 *   label = @Translation("Search"),
 *   uri_paths = {
 *     "canonical" = "/openstory_search/{word}"
 *   }
 * )
 */
class SearchResource extends ResourceBase {

  /**
   * A current user instance.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * Constructs a Drupal\rest\Plugin\ResourceBase object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param array $serializer_formats
   *   The available serialization formats.
   * @param \Psr\Log\LoggerInterface $logger
   *   A logger instance.
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   A current user instance.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    array $serializer_formats,
    LoggerInterface $logger,
    AccountProxyInterface $current_user) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $serializer_formats, $logger);
    $this->currentUser = $current_user;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->getParameter('serializer.formats'),
      $container->get('logger.factory')->get('custom_rest'),
      $container->get('current_user')
    );
  }

  /**
   * Responds to entity GET requests.
   *
   * @param string $word
   *   The word searched.
   *
   * @return \Drupal\rest\ResourceResponse
   *   The HTTP response object.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\HttpException
   */
  public function get($word = '') {
    $indexes = Index::loadMultiple();
    $allResults = [];
    if (!empty($word)) {
      // Search word in all indexes enabled.
      foreach ($indexes as $value) {
        $index = Index::load($value->id());

        if ($index->status()) {
          $query = $index->query();
          $parse_mode = \Drupal::service('plugin.manager.search_api.parse_mode')->createInstance('direct');
          $query->setParseMode($parse_mode);
          // Search word using search api method.
          $query->keys($word);
          $results = $query->execute();
          $resultItems = $results->getResultItems();
          if (!empty($resultItems)) {
            foreach ($resultItems as $itemId => $itemValue) {
              $pattern = '/(?<=:).*?(?=:)/';
              preg_match($pattern, $itemId, $matches);
              $args = explode('/', $matches[0]);
              $item = [];
              $item['entity_type'] = $args[0];
              $item['entity_id'] = $args[1];
              $allResults[] = $item;
            }
          }
        }
      }
      if (!empty($allResults)) {
        $response = new ResourceResponse($allResults, 200);
      }
      else {
        $response = new ResourceResponse(t('Word not found'), 200);
      }
    }
    else {
      $response = new ResourceResponse(t('Please provide a word.'), 200);
    }

    return $response;
  }

}
