<?php

namespace Drupal\openstory\Plugin\rest\resource;

use Drupal\node\Entity\Node;
use Drupal\rest\Plugin\ResourceBase;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\rest\ResourceResponse;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a resource to update the metatag field for nodes.
 *
 * @RestResource(
 *   id = "update_metatag",
 *   label = @Translation("Update metatag field for nodes"),
 *   uri_paths = {
 *     "canonical" = "/update_metatag/{nid}"
 *   }
 * )
 */
class UpdateMetatagResource extends ResourceBase {

  /**
   * A current user instance.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * Constructs a Drupal\rest\Plugin\ResourceBase object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param array $serializer_formats
   *   The available serialization formats.
   * @param \Psr\Log\LoggerInterface $logger
   *   A logger instance.
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   A current user instance.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    array $serializer_formats,
    LoggerInterface $logger,
    AccountProxyInterface $current_user) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $serializer_formats, $logger);
    $this->currentUser = $current_user;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->getParameter('serializer.formats'),
      $container->get('logger.factory')->get('custom_rest'),
      $container->get('current_user')
    );
  }

  /**
   * Responds to entity PATCH requests and returns a node.
   *
   * @param string $nid
   *   The node id.
   * @param string $data
   *   Data.
   *
   * @return \Drupal\rest\ModifiedResourceResponse
   *   The HTTP response object.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\HttpException
   */
  public function patch($nid, $data) {
    if (!empty($data)) {
      // Node load.
      $node = Node::load($nid);
      // Check if node is valid.
      if (!empty($node)) {
        $fields = $node->getFields();
        foreach ($fields as $key => $value) {
          // Check only fields.
          if (strpos($key, 'field_') !== FALSE) {
            $fieldDefinition = $fields[$key]->getDataDefinition();
            if (!empty($fieldDefinition) && $fieldDefinition->getType() == 'metatag') {
              // Set components of metatag.
              $node->set($key, serialize($data));
              $node->save();
            }
          }
        }
        $result = new ResourceResponse($node, 200);
        $result->addCacheableDependency($nid);
        return $result;
      }
      else {
        throw new BadRequestHttpException('Please provide a valid node id.');
      }
    }
    else {
      throw new BadRequestHttpException('Please provide the components of metatag that you want to update.');
    }
  }

}
