<?php

namespace Drupal\openstory\Plugin\views\style;

use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\field\Entity\FieldStorageConfig;
use Drupal\file\Entity\File;
use Drupal\openstory\OpenStoryViewPager;
use Drupal\rest\Plugin\views\style\Serializer;
use Drupal\user\Entity\User;
use Drupal\views\Plugin\views\display\Page;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Serializer\SerializerInterface;

/**
 * Create new serializer that will return the name of a user.
 *
 * @ingroup views_style_plugins
 *
 * @ViewsStyle(
 *   id = "openstory_entity_serializer",
 *   title = @Translation("OpenStory Entity Serializer"),
 *   help = @Translation("OpenStory entity serializer for the rest export"),
 *   display_types = {"data"}
 * )
 */
class OsEntitySerializer extends Serializer {

  /**
   * The view pager.
   *
   * @var \Drupal\openstory\OpenStoryViewPager
   */
  protected $viewPager;

  /**
   * The entity repository service.
   *
   * @var \Drupal\Core\Entity\EntityRepositoryInterface
   */
  protected $entityRepository;

  /**
   * Constructs a Plugin object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Symfony\Component\Serializer\SerializerInterface $serializer
   *   The serializer interface.
   * @param array $serializer_formats
   *   The serializer format.
   * @param array $serializer_format_providers
   *   The serializer format providers.
   * @param \Drupal\openstory\OpenStoryViewPager $viewPager
   *   The serializer format providers.
   * @param \Drupal\Core\Entity\EntityRepositoryInterface $entityRepository
   *   The entity repository service.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, SerializerInterface $serializer, array $serializer_formats, array $serializer_format_providers, OpenStoryViewPager $viewPager, EntityRepositoryInterface $entityRepository) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $serializer, $serializer_formats, $serializer_format_providers);
    $this->viewPager = $viewPager;
    $this->entityRepository = $entityRepository;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('serializer'),
      $container->getParameter('serializer.formats'),
      $container->getParameter('serializer.format_providers'),
      $container->get('openstory_view_pager'),
      $container->get('entity.repository')
    );
  }

  /**
   * Get name of a user.
   */
  public function render() {
    $rows = [];
    foreach ($this->view->result as $row_index => $row) {
      $this->view->row_index = $row_index;
      $renderedRow = $this->view->rowPlugin->render($row);
      $images = [];
      foreach ($renderedRow as $item) {
        $fieldDefinition = $item->getFieldDefinition();
        if ($fieldDefinition->getType() == 'image') {
          $itemValue = $item->getValue();
          $imageSettings = $fieldDefinition->getSettings();
          if (!empty($itemValue)) {
            if (!empty($itemValue[0]) && !empty($itemValue[0]['target_id'])) {
              $fileId = $itemValue[0]['target_id'];
              $file = File::load($fileId);
              $uri = $file->uri;
              if ($uri) {
                $uriValue = $uri->getValue();
                if ($uriValue[0] && $uriValue[0]['value']) {
                  // Need only the first picture.
                  if (empty($images['osContentPicture'])) {
                    $images['osContentPicture'] = ['id' => $fileId, 'url' => file_create_url($uriValue[0]['value'])];
                  }
                }
              }
            }
          }
          elseif (!empty($imageSettings) && !empty($imageSettings['default_image']) && !empty($imageSettings['default_image']['uuid'])) {
            $fileUuid = $imageSettings['default_image']['uuid'];
            $images['osContentPicture'] = $this->getFileProperties($fileUuid);
          }
          else {
            // Default value set.
            $fieldStorageConfig = FieldStorageConfig::loadByName($fieldDefinition->getTargetEntityTypeId(), $fieldDefinition->getName());
            $fileUuid = $fieldStorageConfig->getSetting('default_image')['uuid'];
            $images['osContentPicture'] = $this->getFileProperties($fileUuid);
          }
        }
      }
      $currentRow = $renderedRow->toArray();
      if (is_array($currentRow)) {
        if (array_key_exists('uri', $currentRow)) {
          $currentUri = $currentRow['uri'][0]['value'];
          $currentRow['uri'][0]['value'] = file_create_url($currentUri);
        }
      }
      if (!empty($renderedRow->get('uid'))) {
        $uid = $renderedRow->get('uid')->getValue();
        if (!empty($uid[0]) && !empty($uid[0]['target_id'])) {
          $user = User::load($uid[0]['target_id']);
          if (!empty($user)) {
            if ($user->get('user_picture')) {
              $userPicture = $user->get('user_picture');
              $picturePath = '';
              if ($userPicture) {
                $pictureId = $userPicture->getValue();
                if ($pictureId && $pictureId[0] && $pictureId[0]['target_id']) {
                  $file = File::load($pictureId[0]['target_id']);
                  $uri = $file->uri;
                  if ($uri) {
                    $uriValue = $uri->getValue();
                    if ($uriValue[0] && $uriValue[0]['value']) {
                      $picturePath = file_create_url($uriValue[0]['value']);
                    }
                  }
                }
              }
            }
            $currentRow['username']['name'] = $user->getAccountName();
            if ($picturePath) {
              $currentRow['username']['userPicture'] = $picturePath;
            }
            else {
              $currentRow['username']['userPicture'] = '';
            }
          }
          else {
            $currentRow['username']['name'] = 'Anonymous';
            $currentRow['username']['userPicture'] = '';
          }
        }
      }

      // If there is no picture, put empty id and url.
      if (empty($images['osContentPicture'])) {
        $currentRow['osContentPicture'] = ['id' => '', 'url' => ''];
      }
      else {
        $currentRow['osContentPicture'] = $images['osContentPicture'];
      }
      $rows[] = $currentRow;
    }

    unset($this->view->row_index);

    // Get the content type configured in the display or fallback to the
    // default.
    if ((empty($this->view->live_preview))) {
      $displayHandler = $this->displayHandler;
      if ($displayHandler instanceof Page) {
        $content_type = !empty($this->options['formats']) ? reset($this->options['formats']) : 'json';
      }
      else {
        $content_type = $this->displayHandler->getContentType();
      }
    }
    else {
      $content_type = !empty($this->options['formats']) ? reset($this->options['formats']) : 'json';
    }

    $result = $this->viewPager->getViewResult($this->view, $rows);

    return $this->serializer->serialize($result, $content_type, ['views_style_plugin' => $this]);
  }

  /**
   * Get fid and url of an image.
   *
   * @param string $fileUuid
   *   The file uuid.
   *
   * @return array
   *   Array with image fid and url.
   */
  private function getFileProperties($fileUuid) {
    if (!empty($fileUuid)) {
      $entity = $this->entityRepository->loadEntityByUuid('file', $fileUuid);
      $fid = $entity->get('fid')->getValue();
      $uri = $entity->get('uri')->getValue();
      if ($fid[0] && $fid[0]['value']) {
        $fidValue = $fid[0]['value'];
      }
      if ($uri[0] && $uri[0]['value']) {
        $uriValue = $uri[0]['value'];
      }
      return ['id' => !empty($fidValue) ? $fidValue : '', 'url' => !empty($uriValue) ? file_create_url($uriValue) : ''];
    }
  }

}
